cbuffer cbPerDraw : register(b0)
{
	float4x4 tVP : LAYERVIEWPROJECTION;
};

cbuffer cbPerObj : register( b1 )
{
	float4x4 tW : WORLD;
	float Alpha <float uimin=0.0; float uimax=1.0;> = 1; 
	float4 cAmb <bool color=true;String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
	float4x4 tColor <string uiname="Color Transform";>;
};

cbuffer cbTextureData : register(b2)
{
	float4x4 tTex <string uiname="Texture Transform"; bool uvspace=true; >;
};


struct vsInputTextured
{
    float4 posObject : POSITION;
		float3 norm : NORMAL;
	float4 uv: TEXCOORD0;
};

struct psInputTextured
{
    float4 pos : SV_Position;
	float3 norm : NORMAL;
    float4 uv: TEXCOORD0;
};

Texture2D inputTexture <string uiname="Texture";>;
Texture2D ControlTexture <string uiname="Control";>;

SamplerState linearSampler <string uiname="Sampler State";>
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

psInputTextured VS(vsInputTextured input)
{
	psInputTextured output;
	output.pos = mul(input.posObject,mul(tW,tVP));
	output.norm = input.norm;
	output.uv = mul(input.uv, tTex);
	return output;
}

[maxvertexcount(3)]
void GS(triangle psInputTextured input[3], inout TriangleStream<psInputTextured> gsout)
{
	//float3 ce = (p1+p2+p3)/3.0f;
	
	psInputTextured elem = (psInputTextured)0;
	elem.pos.w=1.0;
	
	float3 p1 = input[0].pos.xyz;
	float3 p2 = input[1].pos.xyz;
	float3 p3 = input[2].pos.xyz;
	
	
	
	float3 faceEdgeA = p2 - p1;
    float3 faceEdgeB = p1 - p3;
    float3 norm = cross(faceEdgeB, faceEdgeA);
	norm = normalize(norm);	
	
	float4 col0 = ControlTexture.SampleLevel(linearSampler,input[0].uv.xy,0);
	float4 col1 = ControlTexture.SampleLevel(linearSampler,input[1].uv.xy,0);
	float4 col2 = ControlTexture.SampleLevel(linearSampler,input[2].uv.xy,0);
	
	float4 average = (col0 + col1 + col2)/3;
	
	float amt = average.r;
	


	elem.pos.xyz = input[0].pos.xyz+(norm*amt);
	elem.pos.w = input[0].pos.w;
	elem.norm = input[0].norm;
	elem.uv = input[0].uv;
	gsout.Append(elem);

	elem.pos.xyz = input[1].pos.xyz+(norm*amt);
	elem.pos.w = input[1].pos.w;
	elem.norm = input[1].norm;
	elem.uv = input[1].uv;
	gsout.Append(elem);
	
	elem.pos.xyz = input[2].pos.xyz+(norm*amt);
	elem.pos.w = input[2].pos.w;
	elem.norm = input[2].norm;
	elem.uv = input[2].uv;
	gsout.Append(elem);

	gsout.RestartStrip();
}

float4 PS(psInputTextured input): SV_Target
{
    float4 col = inputTexture.Sample(linearSampler,input.uv.xy) * cAmb;
	col = mul(col, tColor);
	col.a *= Alpha;
    return col;
}

technique11 Constant
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetGeometryShader ( CompileShader( gs_5_0, GS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}